@@ -1,6 +1,6 @@
-# $Id: Request.pm 327 2008-05-28 17:33:06Z rcaputo $
-
 package POE::Component::Client::HTTP::Request;
+# vim: ts=2 sw=2 expandtab
+
 use strict;
 use warnings;
 
@@ -27,7 +27,7 @@ use constant DEBUG => 0;
 use constant REQ_ID            =>  0;
 use constant REQ_POSTBACK      =>  1;
 use constant REQ_CONNECTION    =>  2;
-use constant REQ_REQUEST       =>  3;
+use constant REQ_HTTP_REQUEST  =>  3;
 use constant REQ_STATE         =>  4;
 use constant REQ_RESPONSE      =>  5;
 use constant REQ_BUFFER        =>  6;
@@ -41,15 +41,15 @@ use constant REQ_HISTORY       => 14;
 use constant REQ_START_TIME    => 15;
 use constant REQ_FACTORY       => 16;
 use constant REQ_CONN_ID       => 17;
+use constant REQ_PEERNAME      => 18;
 
-use constant RS_CONNECT        => 0x01;
-use constant RS_SENDING        => 0x02;
-use constant RS_IN_HEAD        => 0x04;
-use constant RS_REDIRECTED     => 0x08;
-use constant RS_IN_CONTENT     => 0x20;
-use constant RS_DONE           => 0x40;
-use constant RS_POSTED         => 0x80;
-
+use constant RS_CONNECT        => 0x01; # establishing a connection
+use constant RS_SENDING        => 0x02; # sending request to server
+use constant RS_IN_HEAD        => 0x04; # waiting for or receiving headers
+use constant RS_REDIRECTED     => 0x08; # request has been redirected
+use constant RS_IN_CONTENT     => 0x20; # waiting for or receiving content
+use constant RS_DONE           => 0x40; # received full content
+use constant RS_POSTED         => 0x80; # we have posted back a response
 
 sub import {
   my ($class) = shift;
@@ -60,10 +60,10 @@ sub import {
     if ($tag eq ':fields') {
       foreach my $sub (
         qw(
-          REQ_ID REQ_POSTBACK REQ_CONNECTION REQ_REQUEST REQ_STATE
+          REQ_ID REQ_POSTBACK REQ_CONNECTION REQ_HTTP_REQUEST REQ_STATE
           REQ_RESPONSE REQ_BUFFER REQ_OCTETS_GOT REQ_TIMER
           REQ_PROG_POSTBACK REQ_USING_PROXY REQ_HOST REQ_PORT
-          REQ_HISTORY REQ_START_TIME REQ_CONN_ID
+          REQ_HISTORY REQ_START_TIME REQ_CONN_ID REQ_PEERNAME
         )
       ) {
         no strict 'refs';
@@ -143,7 +143,7 @@ sub new {
     $request_id,        # REQ_ID
     $postback,          # REQ_POSTBACK
     undef,              # REQ_CONNECTION
-    $http_request,      # REQ_REQUEST
+    $http_request,      # REQ_HTTP_REQUEST
     RS_CONNECT,         # REQ_STATE
     undef,              # REQ_RESPONSE
     '',                 # REQ_BUFFER
@@ -158,6 +158,7 @@ sub new {
     time(),             # REQ_START_TIME
     $factory,           # REQ_FACTORY
     undef,              # REQ_CONN_ID
+    undef,              # REQ_PEERNAME
   ];
   return bless $self, $class;
 }
@@ -219,7 +220,7 @@ sub add_eof {
   # however.
 
   if (
-    $self->[REQ_REQUEST]->method() ne "HEAD" and
+    $self->[REQ_HTTP_REQUEST]->method() ne "HEAD" and
     defined $self->[REQ_RESPONSE]->content_length and
     not defined $self->[REQ_RESPONSE]->header("Transfer-Encoding") and
     $self->[REQ_OCTETS_GOT] < $self->[REQ_RESPONSE]->content_length
@@ -228,6 +229,7 @@ sub add_eof {
       "got " . $self->[REQ_OCTETS_GOT] . " of " .
       $self->[REQ_RESPONSE]->content_length
     );
+
     $self->error(
       400,
       "incomplete response b " . $self->[REQ_ID] . ".  Wanted " .
@@ -259,23 +261,29 @@ sub add_content {
   # therein.  If it's done the former, then we're safe.  Otherwise
   # we also need to C<use bytes>.
   # TODO: write test(s) for this.
+
   my $this_chunk_length = length($self->[REQ_BUFFER]);
   $self->[REQ_OCTETS_GOT] += $this_chunk_length;
 
-  my $max = $self->[REQ_FACTORY]->max_response_size;
+  my $max = $self->[REQ_FACTORY]->max_response_size();
 
   DEBUG and warn(
     "REQ: request ", $self->ID,
     " received $self->[REQ_OCTETS_GOT] bytes; maximum is $max"
   );
 
+  # Fail if we've gone over the maximum content size to return.
   if (defined $max and $self->[REQ_OCTETS_GOT] > $max) {
-    # We've gone over the maximum content size to return.  Chop it # back.
-    my $over = $self->[REQ_OCTETS_GOT] - $max;
-    $self->[REQ_OCTETS_GOT] -= $over;
-    substr($self->[REQ_BUFFER], -$over) = "";
-    #TODO: ??
-    #$self->[REQ_STATE] |= RS_DONE;
+    $self->error(
+      406,
+      "Response content is longer than specified MaxSize of $max.  " .
+      "Use range requests to retrieve specific amounts of content."
+    );
+
+    $self->[REQ_STATE] |= RS_DONE;
+    $self->[REQ_STATE] &= ~RS_IN_CONTENT;
+    $self->[REQ_CONNECTION]->close();
+    return 1;
   }
 
   # keep this for the progress callback (it gets cleared in return_response
@@ -300,16 +308,6 @@ sub add_content {
     );
   };
 
-  if (defined $max and $self->[REQ_OCTETS_GOT] >= $max) {
-    DEBUG and warn(
-      "REQ: request ", $self->ID, " has a full response... moving to done."
-    );
-    $self->[REQ_STATE] |= RS_DONE;
-    $self->[REQ_STATE] &= ~RS_IN_CONTENT;
-    $self->[REQ_CONNECTION]->close();
-    return 1;
-  }
-
   if ($self->[REQ_RESPONSE]->content_length) {
 
     # Report back progress
@@ -331,6 +329,7 @@ sub add_content {
       return 1;
     }
   }
+
   return 0;
 }
 
@@ -447,7 +446,7 @@ sub check_redirect {
     DEBUG and warn "RED: Too much redirection";
   }
   else { # All fine, yield new request and mark this disabled.
-    my $newrequest = $self->[REQ_REQUEST]->clone();
+    my $newrequest = $self->[REQ_HTTP_REQUEST]->clone();
 
     # Sanitize new request per rt #30400.
     # TODO - What other headers are security risks?
@@ -469,7 +468,7 @@ sub send_to_wheel {
 
   $self->[REQ_STATE] = RS_SENDING;
 
-  my $http_request = $self->[REQ_REQUEST];
+  my $http_request = $self->[REQ_HTTP_REQUEST];
 
   # MEXNIX 2002-06-01: Check for proxy.  Request query is a bit
   # different...
@@ -535,12 +534,13 @@ sub error {
     . "<BODY>$nl"
     . "<H1>Error: $http_msg</H1>$nl"
     . "$message$nl"
+    . "<small>This is a client error, not a server error.</small>$nl"
     . "</BODY>$nl"
     . "</HTML>$nl"
   );
 
   $r->content ($m);
-  $r->request ($self->[REQ_REQUEST]);
+  $r->request ($self->[REQ_HTTP_REQUEST]);
   $self->[REQ_POSTBACK]->($r);
   $self->[REQ_STATE] |= RS_POSTED;
 }
@@ -563,7 +563,7 @@ sub port { shift->[REQ_PORT] }
 sub scheme {
   my $self = shift;
 
-  $self->[REQ_USING_PROXY] ? 'http' : $self->[REQ_REQUEST]->uri->scheme;
+  $self->[REQ_USING_PROXY] ? 'http' : $self->[REQ_HTTP_REQUEST]->uri->scheme;
 }
 
 sub DESTROY {
